@@ -368,9 +368,7 @@ static void
 ecc_mul_binary (const struct ecc_curve *ecc,
 		struct ecc_point *r, const mpz_t n, const struct ecc_point *p)
 {
-  /* Avoid the mp_bitcnt_t type for compatibility with older GMP
-     versions. */
-  unsigned k;
+  mp_bitcnt_t k;
 
   assert (r != p);
   assert (mpz_sgn (n) > 0);
