@@ -53,13 +53,8 @@ equal_h (const struct ecc_modulo *p,
 #define t0 scratch
 #define t1 (scratch + p->size)
 
-  ecc_mod_mul (p, t0, x1, z2, t0);
-  if (mpn_cmp (t0, p->m, p->size) >= 0)
-    mpn_sub_n (t0, t0, p->m, p->size);
-
-  ecc_mod_mul (p, t1, x2, z1, t1);
-  if (mpn_cmp (t1, p->m, p->size) >= 0)
-    mpn_sub_n (t1, t1, p->m, p->size);
+  ecc_mod_mul_canonical (p, t0, x1, z2, t0);
+  ecc_mod_mul_canonical (p, t1, x2, z1, t1);
 
   return mpn_cmp (t0, t1, p->size) == 0;
 
@@ -101,6 +96,10 @@ _eddsa_verify (const struct ecc_curve *ecc,
   if (!_eddsa_decompress (ecc, R, signature, R+2*ecc->p.size))
     return 0;
 
+  /* For Ed448, this conversion reads only the significant 56 octets
+     of the encoding of S. By RFC 8032, S is encoded with a redundant
+     always-zero byte at the end, which should be checked by the
+     caller. */
   mpn_set_base256_le (sp, ecc->q.size, signature + nbytes, nbytes);
   /* Check that s < q */
   if (mpn_cmp (sp, ecc->q.m, ecc->q.size) >= 0)
@@ -110,7 +109,7 @@ _eddsa_verify (const struct ecc_curve *ecc,
   eddsa->update (ctx, nbytes, signature);
   eddsa->update (ctx, nbytes, pub);
   eddsa->update (ctx, length, msg);
-  eddsa->digest (ctx, 2*nbytes, hash);
+  eddsa->digest (ctx, hash);
   _eddsa_hash (&ecc->q, hp, 2*nbytes, hash);
 
   /* Compute h A + R - s G, which should be the neutral point */
