@@ -145,10 +145,6 @@ DECLARE_FAT_FUNC(_nettle_aes_decrypt, aes_crypt_internal_func)
 DECLARE_FAT_FUNC_VAR(aes_decrypt, aes_crypt_internal_func, arm)
 DECLARE_FAT_FUNC_VAR(aes_decrypt, aes_crypt_internal_func, armv6)
 
-DECLARE_FAT_FUNC(_nettle_salsa20_core, salsa20_core_func)
-DECLARE_FAT_FUNC_VAR(salsa20_core, salsa20_core_func, c)
-DECLARE_FAT_FUNC_VAR(salsa20_core, salsa20_core_func, neon)
-
 DECLARE_FAT_FUNC(_nettle_salsa20_crypt, salsa20_crypt_func)
 DECLARE_FAT_FUNC_VAR(salsa20_crypt, salsa20_crypt_func, 1core)
 DECLARE_FAT_FUNC_VAR(salsa20_crypt, salsa20_crypt_func, 2core)
@@ -157,9 +153,9 @@ DECLARE_FAT_FUNC(nettle_sha1_compress, sha1_compress_func)
 DECLARE_FAT_FUNC_VAR(sha1_compress, sha1_compress_func, c)
 DECLARE_FAT_FUNC_VAR(sha1_compress, sha1_compress_func, armv6)
 
-DECLARE_FAT_FUNC(_nettle_sha256_compress, sha256_compress_func)
-DECLARE_FAT_FUNC_VAR(sha256_compress, sha256_compress_func, c)
-DECLARE_FAT_FUNC_VAR(sha256_compress, sha256_compress_func, armv6)
+DECLARE_FAT_FUNC(_nettle_sha256_compress_n, sha256_compress_n_func)
+DECLARE_FAT_FUNC_VAR(sha256_compress_n, sha256_compress_n_func, c)
+DECLARE_FAT_FUNC_VAR(sha256_compress_n, sha256_compress_n_func, armv6)
 
 DECLARE_FAT_FUNC(_nettle_sha512_compress, sha512_compress_func)
 DECLARE_FAT_FUNC_VAR(sha512_compress, sha512_compress_func, c)
@@ -170,16 +166,12 @@ DECLARE_FAT_FUNC_VAR(sha3_permute, sha3_permute_func, c)
 DECLARE_FAT_FUNC_VAR(sha3_permute, sha3_permute_func, neon)
 
 DECLARE_FAT_FUNC(_nettle_umac_nh, umac_nh_func)
-DECLARE_FAT_FUNC_VAR(umac_nh, umac_nh_func, c);
-DECLARE_FAT_FUNC_VAR(umac_nh, umac_nh_func, neon);
+DECLARE_FAT_FUNC_VAR(umac_nh, umac_nh_func, c)
+DECLARE_FAT_FUNC_VAR(umac_nh, umac_nh_func, neon)
 
 DECLARE_FAT_FUNC(_nettle_umac_nh_n, umac_nh_n_func)
-DECLARE_FAT_FUNC_VAR(umac_nh_n, umac_nh_n_func, c);
-DECLARE_FAT_FUNC_VAR(umac_nh_n, umac_nh_n_func, neon);
-
-DECLARE_FAT_FUNC(_nettle_chacha_core, chacha_core_func)
-DECLARE_FAT_FUNC_VAR(chacha_core, chacha_core_func, c);
-DECLARE_FAT_FUNC_VAR(chacha_core, chacha_core_func, neon);
+DECLARE_FAT_FUNC_VAR(umac_nh_n, umac_nh_n_func, c)
+DECLARE_FAT_FUNC_VAR(umac_nh_n, umac_nh_n_func, neon)
 
 DECLARE_FAT_FUNC(nettle_chacha_crypt, chacha_crypt_func)
 DECLARE_FAT_FUNC_VAR(chacha_crypt, chacha_crypt_func, 1core)
@@ -210,7 +202,7 @@ fat_init (void)
       _nettle_aes_encrypt_vec = _nettle_aes_encrypt_armv6;
       _nettle_aes_decrypt_vec = _nettle_aes_decrypt_armv6;
       nettle_sha1_compress_vec = _nettle_sha1_compress_armv6;
-      _nettle_sha256_compress_vec = _nettle_sha256_compress_armv6;
+      _nettle_sha256_compress_n_vec = _nettle_sha256_compress_n_armv6;
     }
   else
     {
@@ -219,19 +211,17 @@ fat_init (void)
       _nettle_aes_encrypt_vec = _nettle_aes_encrypt_arm;
       _nettle_aes_decrypt_vec = _nettle_aes_decrypt_arm;
       nettle_sha1_compress_vec = _nettle_sha1_compress_c;
-      _nettle_sha256_compress_vec = _nettle_sha256_compress_c;
+      _nettle_sha256_compress_n_vec = _nettle_sha256_compress_n_c;
     }
   if (features.have_neon)
     {
       if (verbose)
 	fprintf (stderr, "libnettle: enabling neon code.\n");
-      _nettle_salsa20_core_vec = _nettle_salsa20_core_neon;
       _nettle_salsa20_crypt_vec = _nettle_salsa20_crypt_2core;
       _nettle_sha512_compress_vec = _nettle_sha512_compress_neon;
       nettle_sha3_permute_vec = _nettle_sha3_permute_neon;
       _nettle_umac_nh_vec = _nettle_umac_nh_neon;
       _nettle_umac_nh_n_vec = _nettle_umac_nh_n_neon;
-      _nettle_chacha_core_vec = _nettle_chacha_core_neon;
       nettle_chacha_crypt_vec = _nettle_chacha_crypt_3core;
       nettle_chacha_crypt32_vec = _nettle_chacha_crypt32_3core;
     }
@@ -239,13 +229,11 @@ fat_init (void)
     {
       if (verbose)
 	fprintf (stderr, "libnettle: not enabling neon code.\n");
-      _nettle_salsa20_core_vec = _nettle_salsa20_core_c;
       _nettle_salsa20_crypt_vec = _nettle_salsa20_crypt_1core;
       _nettle_sha512_compress_vec = _nettle_sha512_compress_c;
       nettle_sha3_permute_vec = _nettle_sha3_permute_c;
       _nettle_umac_nh_vec = _nettle_umac_nh_c;
       _nettle_umac_nh_n_vec = _nettle_umac_nh_n_c;
-      _nettle_chacha_core_vec = _nettle_chacha_core_c;
       nettle_chacha_crypt_vec = _nettle_chacha_crypt_1core;
       nettle_chacha_crypt32_vec = _nettle_chacha_crypt32_1core;
     }
@@ -265,10 +253,6 @@ DEFINE_FAT_FUNC(_nettle_aes_decrypt, void,
 		 const uint8_t *src),
 		(rounds, keys, T, length, dst, src))
 
-DEFINE_FAT_FUNC(_nettle_salsa20_core, void,
-		(uint32_t *dst, const uint32_t *src, unsigned rounds),
-		(dst, src, rounds))
-
 DEFINE_FAT_FUNC(_nettle_salsa20_crypt, void,
 		(struct salsa20_ctx *ctx, unsigned rounds,
 		 size_t length, uint8_t *dst,
@@ -279,9 +263,10 @@ DEFINE_FAT_FUNC(nettle_sha1_compress, void,
 		(uint32_t *state, const uint8_t *input),
 		(state, input))
 
-DEFINE_FAT_FUNC(_nettle_sha256_compress, void,
-		(uint32_t *state, const uint8_t *input, const uint32_t *k),
-		(state, input, k))
+DEFINE_FAT_FUNC(_nettle_sha256_compress_n, const uint8_t *,
+		(uint32_t *state, const uint32_t *k,
+		 size_t blocks, const uint8_t *input),
+		(state, k, blocks, input))
 
 DEFINE_FAT_FUNC(_nettle_sha512_compress, void,
 		(uint64_t *state, const uint8_t *input, const uint64_t *k),
@@ -299,10 +284,6 @@ DEFINE_FAT_FUNC(_nettle_umac_nh_n, void,
 		 unsigned length, const uint8_t *msg),
 		(out, n, key, length, msg))
 
-DEFINE_FAT_FUNC(_nettle_chacha_core, void,
-		(uint32_t *dst, const uint32_t *src, unsigned rounds),
-		(dst, src, rounds))
-
 DEFINE_FAT_FUNC(nettle_chacha_crypt, void,
 		(struct chacha_ctx *ctx,
 		 size_t length,
