@@ -86,7 +86,7 @@ gcm_aes128_decrypt(struct gcm_aes128_ctx *ctx,
 
 void
 gcm_aes128_digest(struct gcm_aes128_ctx *ctx,
-		  size_t length, uint8_t *digest)
+		  uint8_t *digest)
 {
-  GCM_DIGEST(ctx, aes128_encrypt, length, digest);
+  GCM_DIGEST(ctx, aes128_encrypt, digest);
 }
