@@ -40,6 +40,7 @@
 
 #include "aes.h"
 #include "camellia.h"
+#include "sm4.h"
 
 #ifdef __cplusplus
 extern "C" {
@@ -95,15 +96,21 @@ extern "C" {
 #define gcm_camellia256_decrypt nettle_gcm_camellia256_decrypt
 #define gcm_camellia256_digest nettle_gcm_camellia256_digest
 
+#define gcm_sm4_set_key nettle_gcm_sm4_set_key
+#define gcm_sm4_set_iv nettle_gcm_sm4_set_iv
+#define gcm_sm4_update nettle_gcm_sm4_update
+#define gcm_sm4_encrypt nettle_gcm_sm4_encrypt
+#define gcm_sm4_decrypt nettle_gcm_sm4_decrypt
+#define gcm_sm4_digest nettle_gcm_sm4_digest
+
 #define GCM_BLOCK_SIZE 16
 #define GCM_IV_SIZE (GCM_BLOCK_SIZE - 4)
 #define GCM_DIGEST_SIZE 16
-#define GCM_TABLE_BITS 8
 
 /* Hashing subkey */
 struct gcm_key
 {
-  union nettle_block16 h[1 << GCM_TABLE_BITS];
+  union nettle_block16 h[0x80];
 };
 
 /* Per-message state, depending on the iv */
@@ -143,7 +150,7 @@ gcm_decrypt(struct gcm_ctx *ctx, const struct gcm_key *key,
 void
 gcm_digest(struct gcm_ctx *ctx, const struct gcm_key *key,
 	   const void *cipher, nettle_cipher_func *f,
-	   size_t length, uint8_t *digest);
+	   uint8_t *digest);
 
 /* Convenience macrology (not sure how useful it is) */
 /* All-in-one context, with hash subkey, message state, and cipher. */
@@ -151,10 +158,11 @@ gcm_digest(struct gcm_ctx *ctx, const struct gcm_key *key,
   { struct gcm_key key; struct gcm_ctx gcm; type cipher; }
 
 /* NOTE: Avoid using NULL, as we don't include anything defining it. */
-#define GCM_SET_KEY(ctx, set_key, encrypt, key)			\
+#define GCM_SET_KEY(ctx, set_key, encrypt, gcm_key)		\
   do {								\
-    (set_key)(&(ctx)->cipher, (key));				\
-    if (0) (encrypt)(&(ctx)->cipher, 0, (void *)0, (void *)0);	\
+    (set_key)(&(ctx)->cipher, (gcm_key));			\
+    if (0) (encrypt)(&(ctx)->cipher, ~(size_t) 0,		\
+		     (uint8_t *) 0, (const uint8_t *) 0);	\
     gcm_set_key(&(ctx)->key, &(ctx)->cipher,			\
 		(nettle_cipher_func *) (encrypt));		\
   } while (0)
@@ -166,22 +174,25 @@ gcm_digest(struct gcm_ctx *ctx, const struct gcm_key *key,
   gcm_update(&(ctx)->gcm, &(ctx)->key, (length), (data))
 
 #define GCM_ENCRYPT(ctx, encrypt, length, dst, src)			\
-  (0 ? (encrypt)(&(ctx)->cipher, 0, (void *)0, (void *)0)		\
+  (0 ? (encrypt)(&(ctx)->cipher, ~(size_t) 0,				\
+		 (uint8_t *) 0, (const uint8_t *) 0)			\
      : gcm_encrypt(&(ctx)->gcm, &(ctx)->key, &(ctx)->cipher,		\
 		   (nettle_cipher_func *) (encrypt),			\
 		   (length), (dst), (src)))
 
 #define GCM_DECRYPT(ctx, encrypt, length, dst, src)			\
-  (0 ? (encrypt)(&(ctx)->cipher, 0, (void *)0, (void *)0)		\
+  (0 ? (encrypt)(&(ctx)->cipher, ~(size_t) 0,				\
+		 (uint8_t *) 0, (const uint8_t *) 0)			\
      : gcm_decrypt(&(ctx)->gcm,  &(ctx)->key, &(ctx)->cipher,		\
 		   (nettle_cipher_func *) (encrypt),			\
 		   (length), (dst), (src)))
 
-#define GCM_DIGEST(ctx, encrypt, length, digest)			\
-  (0 ? (encrypt)(&(ctx)->cipher, 0, (void *)0, (void *)0)		\
+#define GCM_DIGEST(ctx, encrypt, digest)				\
+  (0 ? (encrypt)(&(ctx)->cipher, ~(size_t) 0,				\
+		 (uint8_t *) 0, (const uint8_t *) 0)			\
      : gcm_digest(&(ctx)->gcm, &(ctx)->key, &(ctx)->cipher,		\
 		  (nettle_cipher_func *) (encrypt),			\
-		  (length), (digest)))
+		  (digest)))
 
 struct gcm_aes128_ctx GCM_CTX(struct aes128_ctx);
 
@@ -207,7 +218,7 @@ gcm_aes128_decrypt(struct gcm_aes128_ctx *ctx,
 
 void
 gcm_aes128_digest(struct gcm_aes128_ctx *ctx,
-		  size_t length, uint8_t *digest);
+		  uint8_t *digest);
 
 struct gcm_aes192_ctx GCM_CTX(struct aes192_ctx);
 
@@ -231,7 +242,7 @@ gcm_aes192_decrypt(struct gcm_aes192_ctx *ctx,
 
 void
 gcm_aes192_digest(struct gcm_aes192_ctx *ctx,
-		  size_t length, uint8_t *digest);
+		  uint8_t *digest);
 
 struct gcm_aes256_ctx GCM_CTX(struct aes256_ctx);
 
@@ -255,33 +266,7 @@ gcm_aes256_decrypt(struct gcm_aes256_ctx *ctx,
 
 void
 gcm_aes256_digest(struct gcm_aes256_ctx *ctx,
-		  size_t length, uint8_t *digest);
-
-/* Old aes interface, for backwards compatibility */
-struct gcm_aes_ctx GCM_CTX(struct aes_ctx);
-
-void
-gcm_aes_set_key(struct gcm_aes_ctx *ctx,
-		size_t length, const uint8_t *key);
-
-void
-gcm_aes_set_iv(struct gcm_aes_ctx *ctx,
-	       size_t length, const uint8_t *iv);
-
-void
-gcm_aes_update(struct gcm_aes_ctx *ctx,
-	       size_t length, const uint8_t *data);
-
-void
-gcm_aes_encrypt(struct gcm_aes_ctx *ctx,
-		size_t length, uint8_t *dst, const uint8_t *src);
-
-void
-gcm_aes_decrypt(struct gcm_aes_ctx *ctx,
-		size_t length, uint8_t *dst, const uint8_t *src);
-
-void
-gcm_aes_digest(struct gcm_aes_ctx *ctx, size_t length, uint8_t *digest);
+		  uint8_t *digest);
 
 
 struct gcm_camellia128_ctx GCM_CTX(struct camellia128_ctx);
@@ -297,7 +282,7 @@ void gcm_camellia128_encrypt(struct gcm_camellia128_ctx *ctx,
 void gcm_camellia128_decrypt(struct gcm_camellia128_ctx *ctx,
 			     size_t length, uint8_t *dst, const uint8_t *src);
 void gcm_camellia128_digest(struct gcm_camellia128_ctx *ctx,
-			    size_t length, uint8_t *digest);
+			    uint8_t *digest);
 
 
 struct gcm_camellia256_ctx GCM_CTX(struct camellia256_ctx);
@@ -313,9 +298,24 @@ void gcm_camellia256_encrypt(struct gcm_camellia256_ctx *ctx,
 void gcm_camellia256_decrypt(struct gcm_camellia256_ctx *ctx,
 			     size_t length, uint8_t *dst, const uint8_t *src);
 void gcm_camellia256_digest(struct gcm_camellia256_ctx *ctx,
-			    size_t length, uint8_t *digest);
+			    uint8_t *digest);
+
+
+struct gcm_sm4_ctx GCM_CTX(struct sm4_ctx);
+
+void gcm_sm4_set_key(struct gcm_sm4_ctx *ctx, const uint8_t *key);
+void gcm_sm4_set_iv(struct gcm_sm4_ctx *ctx,
+		    size_t length, const uint8_t *iv);
+void gcm_sm4_update(struct gcm_sm4_ctx *ctx,
+		    size_t length, const uint8_t *data);
+void gcm_sm4_encrypt(struct gcm_sm4_ctx *ctx,
+		     size_t length, uint8_t *dst, const uint8_t *src);
+void gcm_sm4_decrypt(struct gcm_sm4_ctx *ctx,
+		     size_t length, uint8_t *dst, const uint8_t *src);
+void gcm_sm4_digest(struct gcm_sm4_ctx *ctx,
+		    uint8_t *digest);
+
 
-  
 #ifdef __cplusplus
 }
 #endif
