@@ -82,7 +82,7 @@ pkcs1_rsa_sha512_encode(mpz_t m, size_t key_size, struct sha512_ctx *hash)
 			      SHA512_DIGEST_SIZE);
   if (p)
     {
-      sha512_digest(hash, SHA512_DIGEST_SIZE, p);
+      sha512_digest(hash, p);
       nettle_mpz_set_str_256_u(m, key_size, em);
       TMP_GMP_FREE(em);
       return 1;
