@@ -45,7 +45,7 @@ void
 poly1305_aes_set_key (struct poly1305_aes_ctx *ctx, const uint8_t * key)
 {
   aes128_set_encrypt_key(&ctx->aes, (key));
-  _poly1305_set_key(&ctx->pctx, (key+16));
+  _nettle_poly1305_set_key(&ctx->pctx, (key+16));
   ctx->index = 0;
 }
 
@@ -56,18 +56,17 @@ poly1305_aes_set_nonce (struct poly1305_aes_ctx *ctx,
   memcpy (ctx->nonce, nonce, POLY1305_AES_NONCE_SIZE);
 }
 
-#define COMPRESS(ctx, data) _poly1305_block(&(ctx)->pctx, (data), 1)
-
 void
 poly1305_aes_update (struct poly1305_aes_ctx *ctx,
 		     size_t length, const uint8_t *data)
 {
-  MD_UPDATE (ctx, length, data, COMPRESS, (void) 0);
+  ctx->index = _nettle_poly1305_update (&(ctx)->pctx,
+					ctx->block, ctx->index, length, data);
 }
 
 void
 poly1305_aes_digest (struct poly1305_aes_ctx *ctx,
-		     size_t length, uint8_t *digest)
+		     uint8_t *digest)
 {
   union nettle_block16 s;
   /* final bytes */
@@ -79,12 +78,12 @@ poly1305_aes_digest (struct poly1305_aes_ctx *ctx,
       memset (ctx->block + ctx->index + 1,
 	      0, POLY1305_BLOCK_SIZE - 1 - ctx->index);
 
-      _poly1305_block (&ctx->pctx, ctx->block, 0);
+      _nettle_poly1305_block (&ctx->pctx, ctx->block, 0);
     }
   aes128_encrypt(&ctx->aes, POLY1305_BLOCK_SIZE, s.b, ctx->nonce);
   
-  _poly1305_digest (&ctx->pctx, &s);
-  memcpy (digest, s.b, length);
+  _nettle_poly1305_digest (&ctx->pctx, &s);
+  memcpy (digest, s.b, POLY1305_AES_DIGEST_SIZE);
 
   INCREMENT (16, ctx->nonce);
   ctx->index = 0;
