@@ -88,6 +88,12 @@ extern "C" {
 #define rsa_encrypt nettle_rsa_encrypt
 #define rsa_decrypt nettle_rsa_decrypt
 #define rsa_decrypt_tr nettle_rsa_decrypt_tr
+#define rsa_oaep_sha256_encrypt nettle_rsa_oaep_sha256_encrypt
+#define rsa_oaep_sha256_decrypt nettle_rsa_oaep_sha256_decrypt
+#define rsa_oaep_sha384_encrypt nettle_rsa_oaep_sha384_encrypt
+#define rsa_oaep_sha384_decrypt nettle_rsa_oaep_sha384_decrypt
+#define rsa_oaep_sha512_encrypt nettle_rsa_oaep_sha512_encrypt
+#define rsa_oaep_sha512_decrypt nettle_rsa_oaep_sha512_decrypt
 #define rsa_sec_decrypt nettle_rsa_sec_decrypt
 #define rsa_compute_root nettle_rsa_compute_root
 #define rsa_compute_root_tr nettle_rsa_compute_root_tr
@@ -98,7 +104,6 @@ extern "C" {
 #define rsa_public_key_from_der_iterator nettle_rsa_public_key_from_der_iterator
 #define rsa_private_key_from_der_iterator nettle_rsa_private_key_from_der_iterator
 #define rsa_keypair_from_der nettle_rsa_keypair_from_der
-#define rsa_keypair_to_openpgp nettle_rsa_keypair_to_openpgp
 
 /* This limit is somewhat arbitrary. Technically, the smallest modulo
    which makes sense at all is 15 = 3*5, phi(15) = 8, size 4 bits. But
@@ -389,8 +394,6 @@ rsa_pss_sha512_verify_digest(const struct rsa_public_key *key,
 
 
 /* RSA encryption, using PKCS#1 */
-/* These functions uses the v1.5 padding. What should the v2 (OAEP)
- * functions be called? */
 
 /* Returns 1 on success, 0 on failure, which happens if the
  * message is too long for the key. */
@@ -428,13 +431,61 @@ rsa_sec_decrypt(const struct rsa_public_key *pub,
 	        size_t length, uint8_t *message,
 	        const mpz_t gibberish);
 
-/* Compute x, the e:th root of m. Calling it with x == m is allowed. */
+/* RSA encryption, using OAEP */
+
+int
+rsa_oaep_sha256_encrypt (const struct rsa_public_key *key,
+			 void *random_ctx, nettle_random_func *random,
+			 size_t label_length, const uint8_t *label,
+			 size_t length, const uint8_t * message,
+			 uint8_t *ciphertext);
+
+int
+rsa_oaep_sha256_decrypt (const struct rsa_public_key *pub,
+			 const struct rsa_private_key *key,
+			 void *random_ctx, nettle_random_func *random,
+			 size_t label_length, const uint8_t *label,
+			 size_t *length, uint8_t *message,
+			 const uint8_t *ciphertext);
+
+int
+rsa_oaep_sha384_encrypt (const struct rsa_public_key *key,
+			 void *random_ctx, nettle_random_func *random,
+			 size_t label_length, const uint8_t *label,
+			 size_t length, const uint8_t * message,
+			 uint8_t *ciphertext);
+
+int
+rsa_oaep_sha384_decrypt (const struct rsa_public_key *pub,
+			 const struct rsa_private_key *key,
+			 void *random_ctx, nettle_random_func *random,
+			 size_t label_length, const uint8_t *label,
+			 size_t *length, uint8_t *message,
+			 const uint8_t *ciphertext);
+
+int
+rsa_oaep_sha512_encrypt (const struct rsa_public_key *key,
+			 void *random_ctx, nettle_random_func *random,
+			 size_t label_length, const uint8_t *label,
+			 size_t length, const uint8_t *message,
+			 uint8_t *ciphertext);
+
+int
+rsa_oaep_sha512_decrypt (const struct rsa_public_key *pub,
+			 const struct rsa_private_key *key,
+			 void *random_ctx, nettle_random_func *random,
+			 size_t label_length, const uint8_t *label,
+			 size_t *length, uint8_t *message,
+			 const uint8_t *ciphertext);
+
+/* Compute x, the e:th root of m. Calling it with x == m is allowed.
+   It is required that 0 <= m < n. */
 void
 rsa_compute_root(const struct rsa_private_key *key,
 		 mpz_t x, const mpz_t m);
 
 /* Safer variant, using RSA blinding, and checking the result after
-   CRT. */
+   CRT. It is required that 0 <= m < n. */
 int
 rsa_compute_root_tr(const struct rsa_public_key *pub,
 		    const struct rsa_private_key *key,
@@ -521,15 +572,6 @@ rsa_keypair_from_der(struct rsa_public_key *pub,
 		     unsigned limit, 
 		     size_t length, const uint8_t *data);
 
-/* OpenPGP format. Experimental interface, subject to change. */
-int
-rsa_keypair_to_openpgp(struct nettle_buffer *buffer,
-		       const struct rsa_public_key *pub,
-		       const struct rsa_private_key *priv,
-		       /* A single user id. NUL-terminated utf8. */
-		       const char *userid);
-
-
 #ifdef __cplusplus
 }
 #endif
