@@ -47,6 +47,7 @@
 #include "salsa20.h"
 #include "salsa20-internal.h"
 
+#include "bswap-internal.h"
 #include "macros.h"
 
 /* For fat builds */
@@ -78,14 +79,6 @@ _nettle_salsa20_core_c(uint32_t *dst, const uint32_t *src, unsigned rounds);
 # define DEBUG(i)
 #endif
 
-#ifdef WORDS_BIGENDIAN
-#define LE_SWAP32(v)				\
-  ((ROTL32(8,  v) & 0x00FF00FFUL) |		\
-   (ROTL32(24, v) & 0xFF00FF00UL))
-#else
-#define LE_SWAP32(v) (v)
-#endif
-
 #define QROUND(x0, x1, x2, x3) do { \
   x1 ^= ROTL32(7, x0 + x3);	    \
   x2 ^= ROTL32(9, x1 + x0);	    \
@@ -121,6 +114,6 @@ _nettle_salsa20_core(uint32_t *dst, const uint32_t *src, unsigned rounds)
   for (i = 0; i < _SALSA20_INPUT_LENGTH; i++)
     {
       uint32_t t = x[i] + src[i];
-      dst[i] = LE_SWAP32 (t);
+      dst[i] = bswap32_if_be (t);
     }
 }
