@@ -1,6 +1,6 @@
-/* sha512-256-meta.c
+/* sha3-256-meta.c
 
-   Copyright (C) 2014 Niels Möller
+   Copyright (C) 2012 Niels Möller
 
    This file is part of GNU Nettle.
 
@@ -35,15 +35,8 @@
 
 #include "nettle-meta.h"
 
-#include "sha2.h"
-
-const struct nettle_hash nettle_sha512_256 =
-  {
-    "sha512_256", sizeof(struct sha512_ctx),
-    SHA512_256_DIGEST_SIZE,
-    SHA512_256_BLOCK_SIZE,
-    (nettle_hash_init_func *) sha512_256_init,
-    (nettle_hash_update_func *) sha512_256_update,
-    (nettle_hash_digest_func *) sha512_256_digest
-  };
+#include "sha3.h"
+#include "sha3-internal.h"
 
+const struct nettle_hash nettle_sha3_256
+= _NETTLE_SHA3_HASH(sha3_256, SHA3_256);
