@@ -50,7 +50,7 @@ test_compare_results(const char *name,
         const struct tstring *adata,
         /* Expected results. */
         const struct tstring *e_clear,
-		const struct tstring *e_cipher,
+        const struct tstring *e_cipher,
         /* Actual results. */
         const void *clear,
         const void *cipher,
@@ -133,7 +133,7 @@ test_cipher_ccm(const struct nettle_cipher *cipher,
     ccm_update(&ccm, ctx, cipher->encrypt, authdata->length, authdata->data);
   }
   ccm_encrypt(&ccm, ctx, cipher->encrypt, cleartext->length, en_data, cleartext->data);
-  ccm_digest(&ccm, ctx, cipher->encrypt, tlength, en_digest);
+  ccm_digest(&ccm, ctx, cipher->encrypt, en_digest);
 
   /* Decrypt using the incremental API. */
   ccm_set_nonce(&ccm, ctx, cipher->encrypt, nonce->length, nonce->data,
@@ -142,7 +142,7 @@ test_cipher_ccm(const struct nettle_cipher *cipher,
     ccm_update(&ccm, ctx, cipher->encrypt, authdata->length, authdata->data);
   }
   ccm_decrypt(&ccm, ctx, cipher->encrypt, cleartext->length, de_data, ciphertext->data);
-  ccm_digest(&ccm, ctx, cipher->encrypt, tlength, de_digest);
+  ccm_digest(&ccm, ctx, cipher->encrypt, de_digest);
 
   /* Compare results using the generic API. */
   test_compare_results("CCM", authdata,
@@ -150,37 +150,43 @@ test_cipher_ccm(const struct nettle_cipher *cipher,
 
   /* Ensure we get the same answers using the all-in-one API. */
   if (repeat <= 1) {
-    int ret;
     memset(de_data, 0, cleartext->length);
     memset(en_data, 0, ciphertext->length);
-    memset(de_digest, 0, sizeof(de_digest));
 
     ccm_encrypt_message(ctx, cipher->encrypt, nonce->length, nonce->data,
 			authdata->length, authdata->data, tlength,
 			ciphertext->length, en_data, cleartext->data);
 
-    ret = ccm_decrypt_message(ctx, cipher->encrypt, nonce->length, nonce->data,
-			      authdata->length, authdata->data, tlength,
-			      cleartext->length, de_data, ciphertext->data);
-
-    if (ret != 1) fprintf(stderr, "ccm_decrypt_message failed to validate message\n");
+    if (!ccm_decrypt_message(ctx, cipher->encrypt, nonce->length, nonce->data,
+			     authdata->length, authdata->data, tlength,
+			     cleartext->length, de_data, ciphertext->data))
+      {
+	fprintf(stderr, "ccm_decrypt_message failed to validate message\n");
+	FAIL();
+      }
     test_compare_results("CCM_MSG", authdata,
 			 cleartext, ciphertext, de_data, en_data, NULL);
 
     /* Ensure that we can detect corrupted message or tag data. */
     if (tlength) {
       en_data[0] ^= 1;
-      ret = ccm_decrypt_message(ctx, cipher->encrypt, nonce->length, nonce->data,
-				authdata->length, authdata->data, tlength,
-				cleartext->length, de_data, en_data);
-      if (ret != 0) fprintf(stderr, "ccm_decrypt_message failed to detect corrupted message\n");
+      if (ccm_decrypt_message(ctx, cipher->encrypt, nonce->length, nonce->data,
+			      authdata->length, authdata->data, tlength,
+			      cleartext->length, de_data, en_data))
+	{
+	  fprintf(stderr, "ccm_decrypt_message failed to detect corrupted message\n");
+	  FAIL();
+	}
     }
     /* Ensure we can detect corrupted adata. */
     if (tlength && authdata->length) {
-      ret = ccm_decrypt_message(ctx, cipher->encrypt, nonce->length, nonce->data,
-				authdata->length-1, authdata->data, tlength,
-				cleartext->length, de_data, ciphertext->data);
-      if (ret != 0) fprintf(stderr, "ccm_decrypt_message failed to detect corrupted message\n");
+      if  (ccm_decrypt_message(ctx, cipher->encrypt, nonce->length, nonce->data,
+			       authdata->length-1, authdata->data, tlength,
+			       cleartext->length, de_data, ciphertext->data))
+	{
+	  fprintf(stderr, "ccm_decrypt_message failed to detect corrupted message\n");
+	  FAIL();
+	}
     }
   }
 
@@ -199,7 +205,7 @@ test_cipher_ccm(const struct nettle_cipher *cipher,
       ccm_aes128_update(&aes, authdata->length, authdata->data);
     }
     ccm_aes128_encrypt(&aes, cleartext->length, en_data, cleartext->data);
-    ccm_aes128_digest(&aes, tlength, en_digest);
+    ccm_aes128_digest(&aes, en_digest);
 
     /* AES-128 decrypt. */
     ccm_aes128_set_nonce(&aes, nonce->length, nonce->data,
@@ -208,10 +214,27 @@ test_cipher_ccm(const struct nettle_cipher *cipher,
       ccm_aes128_update(&aes, authdata->length, authdata->data);
     }
     ccm_aes128_decrypt(&aes, cleartext->length, de_data, ciphertext->data);
-    ccm_aes128_digest(&aes, tlength, de_digest);
+    ccm_aes128_digest(&aes, de_digest);
 
     test_compare_results("CCM_AES_128", authdata,
 			 cleartext, ciphertext, de_data, en_data, de_digest);
+    if (repeat <= 1)
+      {
+	memset(de_data, 0, cleartext->length);
+	memset(en_data, 0, ciphertext->length);
+	ccm_aes128_encrypt_message (ctx, nonce->length, nonce->data,
+				    authdata->length, authdata->data, tlength,
+				    ciphertext->length, en_data, cleartext->data);
+	if (!ccm_aes128_decrypt_message(ctx, nonce->length, nonce->data,
+					authdata->length, authdata->data, tlength,
+					cleartext->length, de_data, ciphertext->data))
+	  {
+	    fprintf(stderr, "ccm_aes128_decrypt_message failed to validate message\n");
+	    FAIL();
+	  }
+	test_compare_results("CCM_AES_128_MSG", authdata,
+			     cleartext, ciphertext, de_data, en_data, NULL);
+      }
   }
   /* TODO: I couldn't find any test cases for CCM-AES-192 */
   if (cipher == &nettle_aes256) {
@@ -228,7 +251,7 @@ test_cipher_ccm(const struct nettle_cipher *cipher,
       ccm_aes256_update(&aes, authdata->length, authdata->data);
     }
     ccm_aes256_encrypt(&aes, cleartext->length, en_data, cleartext->data);
-    ccm_aes256_digest(&aes, tlength, en_digest);
+    ccm_aes256_digest(&aes, en_digest);
 
     /* AES-256 decrypt. */
     ccm_aes256_set_nonce(&aes, nonce->length, nonce->data,
@@ -237,10 +260,27 @@ test_cipher_ccm(const struct nettle_cipher *cipher,
       ccm_aes256_update(&aes, authdata->length, authdata->data);
     }
     ccm_aes256_decrypt(&aes, cleartext->length, de_data, ciphertext->data);
-    ccm_aes256_digest(&aes, tlength, de_digest);
+    ccm_aes256_digest(&aes, de_digest);
 
     test_compare_results("CCM_AES_256", authdata,
 			 cleartext, ciphertext, de_data, en_data, de_digest);
+    if (repeat <= 1)
+      {
+	memset(de_data, 0, cleartext->length);
+	memset(en_data, 0, ciphertext->length);
+	ccm_aes256_encrypt_message (ctx, nonce->length, nonce->data,
+				    authdata->length, authdata->data, tlength,
+				    ciphertext->length, en_data, cleartext->data);
+	if (!ccm_aes256_decrypt_message(ctx, nonce->length, nonce->data,
+					authdata->length, authdata->data, tlength,
+					cleartext->length, de_data, ciphertext->data))
+	  {
+	    fprintf(stderr, "ccm_aes256_decrypt_message failed to validate message\n");
+	    FAIL();
+	  }
+	test_compare_results("CCM_AES_256_MSG", authdata,
+			     cleartext, ciphertext, de_data, en_data, NULL);
+      }
   }
 
   free(ctx);
@@ -593,16 +633,6 @@ test_main(void)
 		  SHEX(""),
 		  SHEX("22 3B C1 EC 84 1A B5 53"));
 
-  /*
-   * C.2.2 MAC data frame
-   */
-  test_cipher_ccm(&nettle_aes128,
-		  SHEX("C0 C1 C2 C3 C4 C5 C6 C7 C8 C9 CA CB CC CD CE CF"),
-		  SHEX("AC DE 48 00 00 00 00 01 00 00 00 05 04"),
-		  SHEX("69 DC 84 21 43 02 00 00 00 00 48 DE AC 01 00 00 00 00 48 DE AC 04 05 00 00 00"), 1,
-		  SHEX("61 62 63 64"),
-		  SHEX("D4 3E 02 2B"));
-
   /*
    * C.2.3 MAC command frame
    */
